/* -*-c-*-
 *
 * $Id$
 *
 * Useful key-management functions
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the Perl interface to Catacomb.
 *
 * Catacomb/Perl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Catacomb/Perl is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Catacomb/Perl; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Header files ------------------------------------------------------*/

#include "catacomb-perl.h"

/*----- Main code ---------------------------------------------------------*/

void keyreport(const char *file, int line, const char *err, void *p)
{
  SV *sv = p;
  dSP;

  if (!SvOK(sv))
    warn("%s:%i: keyfile error: %s", file, line, err);
  else {
    ENTER; SAVETMPS;

    PUSHMARK(SP);
    XPUSHs(sv);
    XPUSHs(sv_2mortal(newSVpv(file, 0)));
    XPUSHs(sv_2mortal(newSViv(line)));
    XPUSHs(sv_2mortal(newSVpv(err, 0)));
    PUTBACK;

    call_method("report", G_DISCARD | G_EVAL | G_KEEPERR);
    if (SvTRUE(ERRSV)) {
      STRLEN len;
      warn("reporter raised error (ignoring): %s", SvPV(ERRSV, len));
    }

    FREETMPS; LEAVE;
  }
}

SV *keyerr(int rc)
{
  SV *sv;
  if (!rc)
    return (&PL_sv_yes);
  sv = perl_get_sv("Catacomb::keyerror", TRUE);
  sv_setiv(sv, rc);
  sv_setpv(sv, key_strerror(rc));
  SvIOK_on(sv);
  return (&PL_sv_undef);
}

const char *keydata_type(Key_Data *kd)
{
  const char *cls;
  if (!kd) return (0);
  switch (kd->e & KF_ENCMASK) {
    case KENC_BINARY: cls = "Catacomb::Key::Data::Binary"; break;
    case KENC_ENCRYPT: cls = "Catacomb::Key::Data::Encrypted"; break;
    case KENC_EC: cls = "Catacomb::Key::Data::EC"; break;
    case KENC_STRING: cls = "Catacomb::Key::Data::String"; break;
    case KENC_STRUCT: cls = "Catacomb::Key::Data::Structured"; break;
    case KENC_MP: cls = "Catacomb::Key::Data::MP"; break;
    default: abort();
  }
  return (cls);
}

void keyfile_dec(Key_File *kf)
{
  kf->ref--;
  if (!kf->ref) {
    key_discard(&kf->kf);
    DESTROY(kf);
  }
}

/*----- That's all, folks -------------------------------------------------*/
